/**
 * @internal
 * @TODO cleaning and optimization
 */
import { messageParser } from "./utils/message-parser.util.js";
import { reallyEmpty } from "./utils/ops.util.js";
import { getValueByStringNotation } from "./utils/obj.util.js";
import { parseStringNotationRules, parseStringRule } from "./utils/rules-parser.util.js";
import * as MessagesProvider from './messages/provider.js';
import * as config from './config.js';
let RulesProvider = {};
let PostRulesProvider = {};
/**
 * registerRules will replace old registered rules
 * @param rules validation rules
 */
export function registerRules(rules) {
    // Object.keys(rules).forEach((rule) => {
    //   RulesProvider[rule] = rules[rule];
    // });
    // return RulesProvider;
    return RulesProvider = rules;
}
/**
 * registerPostRules will replace old post validation rules
 * @param rules post validation rules
 */
export function registerPostRules(rules) {
    return PostRulesProvider = rules;
}
export class ValidatorAbstract {
    /**
     * init validator
     * @param inputs
     * @param rules
     * @param customMessages
     */
    constructor(inputs = {}, rules = {}, customMessages = {}) {
        this.inputs = inputs;
        this.rules = rules;
        this.customMessages = customMessages;
        this.implicitRules = config.get('implicitRules');
        // validation errors collection
        this.errors = {};
        // local custom attributes collection
        this.localNiceNames = {};
        // do we have custom messages?
        this.hasCustomMessages = false;
        // notationMap: any = {};
        // notationVals: any = {};
        // do we have nested rules?
        this.hasNestedRules = false;
        // rules collection
        this.parsedRulesCollection = {};
        this.lang = config.get('lang');
        // post validation rules collection
        this.postRules = [];
        // break rules validation loop when failed
        this.multipleErrors = config.get('multipleErrors');
        // release attribute from futher validation rules
        this.shouldRelease = false;
        this.implicitInputs = {};
        this.inputsAsPerRules = {
            state: {},
            data: {},
        };
        this.hasCustomMessages = Object.keys(customMessages).length > 0;
        this.parse();
    }
    /**
     * @depricated
     * globally should break/bail on failed validation or not
     * @param {boolean} sure
     */
    static bailable(sure) {
        config.set({ multipleErrors: sure === false });
    }
    /**
     * enable/disable multiple errors on current instance only
     * @param {boolean} sure
     */
    bailable(sure) {
        this.multipleErrors = sure === false;
    }
    /**
     * @unused
     * release attribute from rules validation
     */
    release() {
        this.shouldRelease = true;
    }
    /**
     * @since v5
     * @added v5.0.0.beta-6
     * @beta
     * get inputs as per declared rules
     * @returns
     */
    data() {
        return this.inputsAsPerRules.data;
    }
    /**
     * check is instance supports multiple errors or not
     * @returns {boolean}
     */
    isBailable() {
        return this.multipleErrors == false;
    }
    /**
     * allows a custom rule to be added as an implicit rule for the instance only
     * @param {String} ruleName
     */
    addImplicitRule(ruleName) {
        this.implicitRules.push(ruleName);
    }
    addRules(rules = {}) {
        this.rules = rules;
        this.parseRules();
    }
    async postRuleApply(rule) {
        if (rule.rule === 'function') {
            // eslint-disable-next-line no-return-await
            return await rule.handler(this);
        }
        // eslint-disable-next-line no-return-await
        return await PostRulesProvider[rule.rule](rule, this);
    }
    /**
    * add post rule
    *
    * post rule is applied to whole input and is used to check constraints
    * across multiple fields
    *
    * @param {*} rule
   */
    addPostRule(rule) {
        if (typeof rule === 'function') {
            this.postRules.push({
                rule: 'function',
                handler: rule,
            });
            return;
        }
        const ruleArray = rule.split(':', 2);
        const ruleName = ruleArray[0];
        const ruleFields = ruleArray[1].split(','); // there always be a list of fields
        this.postRules.push({
            rule: ruleName,
            params: ruleFields,
        });
    }
    /**
      * add set of post rules
      *
      * @param {string[]} postRulesObj
      */
    addPostRules(postRulesObj) {
        if (!Array.isArray(postRulesObj)) {
            postRulesObj = postRulesObj.split('|');
        }
        postRulesObj.map((rule) => this.addPostRule(rule));
    }
    /**
     * parse provided rules and inputs
     */
    parse() {
        this.parseRules();
        this.parseInputs();
    }
    parseRules() {
        const keys = Object.keys(this.rules);
        const parsed = {};
        for (const key of keys) {
            if (key === '*') {
                this.addPostRules(this.rules[key]);
                continue;
            }
            let rules = this.rules[key];
            if (Array.isArray(rules)) {
                for (const index in rules) {
                    if (typeof rules[index] == 'string') {
                        rules[index] = parseStringRule(RulesProvider, rules[index]);
                    }
                }
            }
            else {
                rules = parseStringNotationRules(RulesProvider, rules);
            }
            rules.sort((obj) => {
                return (this.implicitRules.indexOf(obj.name) >= 0) ? -1 : 1;
            });
            let kobj = parsed;
            let path = '';
            key.split('.').forEach((k, i) => {
                path += k;
                if (!kobj[k]) {
                    kobj[k] = {
                        rules: this.rules[path] ? rules : [],
                        child: {},
                    };
                }
                kobj = kobj[k].child;
                path += '.';
            });
        }
        this.parsedRulesCollection = parsed;
    }
    parseInputs() {
        const obj = {};
        this.inputsAsPerRules = { state: obj, data: obj };
        // if (!this.hasNestedRules) {
        //   return;
        // }
    }
    /**
     * @TODO
     * apply this set of filters to inputs
     * @param filters set of filters
     */
    applyFilters(filters) {
        // future ref
    }
    /**
     * @TODO
    * apply post validation rules
    * @param rules post rules
    */
    applyPostRules(rules) {
        // to-do
    }
    /**
     * validate inputs againest rules
     */
    async validate(inputs) {
        if (inputs) {
            this.inputs = inputs;
            this.parseInputs();
        }
        // console.log(JSON.stringify(this.parsedRulesCollection, null, 2))
        const keys = Object.keys(this.parsedRulesCollection);
        const len = keys.length;
        let i = 0;
        const promises = [];
        for (i; i < len; i += 1) {
            const attrName = keys[i];
            promises.push(this.applyParsedRules(attrName, this.parsedRulesCollection[attrName], this.inputs));
        }
        // post validation rules
        this.postRules.forEach((postRule) => {
            promises.push(this.postRuleApply(postRule));
        });
        await Promise.all(promises);
        return !this.hasErrors();
    }
    /**
     * apply parsed rules
     * @param attrName
     * @param rules
     * @param inputs
     * @param prefix
     * @param state
     * @returns
     */
    async applyParsedRules(attrName, rules, inputs, prefix = '', state = null) {
        state = state || this.inputsAsPerRules.state;
        // array handling
        if (attrName === '*') {
            if (!Array.isArray(inputs)) {
                // this.createAttributeError({
                //     attrName,
                //     attrValue: inputs,
                //     ruleName: 'array',
                //     ruleArgs: [],
                // });
                return;
            }
            let i = 0;
            for (const val of inputs) {
                // console.log(i, rules, [val], `${prefix}${i}`);
                await this.applyParsedRules(i, rules, inputs, prefix, state);
                i += 1;
            }
            return;
        }
        const attrValue = inputs[attrName];
        const info = await this.validateAttribute(attrName, attrValue, rules.rules, { prefix, inputs });
        // if (info.passed) {
        //   console.log('attr', attrName, attrValue, this.inputsAsPerRules.state);
        //   try {
        //     if (state[attrName] == undefined) {
        //       if (typeof attrValue == 'object') {
        //         state = state[attrName] = Array.isArray(attrValue) ? [] : {};
        //       } else {
        //         state = state[attrName] = attrValue;
        //       }
        //     }
        //     //state = state[attrName] = attrValue;
        //   } catch (e) {
        //     console.log('attr', attrName, attrValue, state);
        //     console.info(this.inputsAsPerRules.data);
        //     throw e;
        //   }
        // }
        //console.log(attrName, attrValue, info)
        if (info.empty && info.passed) {
            return;
        }
        // input collection logic
        if (state[attrName] == undefined) {
            state[attrName] = attrValue;
            if (typeof attrValue == 'object') {
                state[attrName] = Array.isArray(attrValue) ? [] : {};
                state = state[attrName];
            }
        }
        for (const cattrName in rules.child) {
            await this.applyParsedRules(cattrName, rules.child[cattrName], attrValue || {}, prefix.length ? `${prefix}${attrName}.` : `${attrName}.`, state);
            //  break;
        }
    }
    /**
     * apply rules
     * @param attrName
     * @param attrValue
     * @param attrRules
     * @param param3
     * @returns
     */
    async validateAttribute(attrName, attrValue, attrRules, { prefix, inputs }) {
        const info = {
            implicit: false,
            implicitFailed: false,
            passed: true,
            empty: reallyEmpty(attrValue),
        };
        let i = 0;
        let len = attrRules.length;
        for (i; i < len; i += 1) {
            const validationRule = attrRules[i];
            const isImplicitRule = this.implicitRules.indexOf(validationRule.name) >= 0;
            if (isImplicitRule) {
                info.implicit = true;
            }
            if (validationRule.name == 'skip') {
                return info;
            }
            if (!isImplicitRule && info.empty) {
                if (this.isBailable()) {
                    return info;
                }
                attrValue = '';
            }
            const passed = await validationRule.handler(attrValue, this, attrName, { path: prefix + attrName, inputs });
            if (passed == -1) {
                continue;
            }
            this.shouldRelease = false;
            if (!passed) {
                info.passed = false;
                this.createAttributeError({
                    attrName: prefix + attrName,
                    attrValue,
                    ruleName: validationRule.name,
                    ruleArgs: validationRule.args,
                });
                if (this.isBailable()) {
                    return info;
                }
            }
        }
        return info;
    }
    /**
    * this will create error object for attribute
    * @param params info object
    */
    createAttributeError(params) {
        const { attrName, ruleName } = params;
        const error = {
            rule: ruleName,
            message: this.createAttributeErrorMessage(params),
        };
        if (this.multipleErrors) {
            if (!this.errors[attrName]) {
                this.errors[attrName] = [];
            }
            // @ts-ignore
            this.errors[attrName].push(error);
            return;
        }
        this.errors[attrName] = error;
    }
    /**
     * this will return parsed error message as per rule or input
     * @param params object with attr and rule name, value, args
     */
    createAttributeErrorMessage(params, useDefaultMessage = true) {
        const { attrName, ruleName, attrValue, ruleArgs } = params;
        const messagesCollection = MessagesProvider.messagesRefByLang(this.lang);
        const defaultMessage = messagesCollection.$default;
        let message;
        // check for local scope messages
        if (this.hasCustomMessages) {
            message = this.customMessages[`${attrName}.${ruleName}`] ||
                this.customMessages[ruleName] ||
                this.customMessages[attrName];
        }
        // not found in local scope, check for global scope
        if (!message) {
            message = (messagesCollection.$custom &&
                messagesCollection.$custom[`${attrName}.${ruleName}`]) ||
                (messagesCollection.$custom && messagesCollection.$custom[attrName]) ||
                messagesCollection[ruleName];
            if (useDefaultMessage && !message) {
                message = defaultMessage;
            }
        }
        let attributeName = attrName;
        let niceName;
        // check if we have nice name in local scope
        if (this.localNiceNames[attrName]) {
            niceName = this.localNiceNames[attrName];
        }
        else if (messagesCollection.$niceNames && messagesCollection.$niceNames[attrName]) {
            // check if we have nice name in global scope
            niceName = messagesCollection.$niceNames[attrName];
        }
        return messageParser({
            message,
            attrName: attributeName,
            niceName,
            ruleName,
            attrValue,
            ruleArgs,
        });
    }
    /**
    * get attribute value by its name
    * @param attr attribute name
    */
    attributeValue(attr) {
        return getValueByStringNotation(this.inputs, attr);
        // return this.inputs[attr] || this.notationVals[attr];
    }
    /**
     * check if attribute has value
     * @param attr attribute name
     */
    doAttributeHasValue(attr) {
        return this.attributeValue(attr) === undefined ? true : false;
    }
    /**
     * does attribute present in given inputs
     * @param attr attribute name
     */
    isAttributePresent(attr) {
        if (this.inputs[attr] != undefined) {
            return true;
        }
        return false;
    }
    /**
     * does we have any dirty/failed input
     */
    hasErrors() {
        return Object.keys(this.errors).length > 0;
    }
    /**
     * get validation errors
     */
    getErrors() {
        return this.errors;
    }
    niceNames(niceNames) {
        this.localNiceNames = niceNames;
    }
}
//# sourceMappingURL=validator-abstract.js.map